// $Id: NCP.h 1327 2005-09-07 17:34:57Z vern $
//
// Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#ifndef ncp_h
#define ncp_h

// A very crude analyzer for NCP (Netware Core Protocol)

#include "NetVar.h"
#include "TCP.h"

#include "ncp_pac.h"

// Create a general NCP_Session class so that it can be used in
// case the RPC conversation is tunneled through other connections,
// e.g., through an SMB session.

class NCP_Session {
public:
	NCP_Session(Connection* conn);
	virtual void Deliver(int is_orig, int len, const u_char* data);

	static bool any_ncp_event()
		{
		return ncp_request || ncp_reply;
		}

protected:
	void DeliverFrame(const BinPAC::ncp_frame* frame);

	Connection* conn;
	int req_frame_type;
	int req_func;
};

class FrameBuffer {
public:
	FrameBuffer(int header_length);
	virtual ~FrameBuffer();

	// Returns true if a frame is ready
	bool Deliver(int& len, const u_char* &data);

	void Reset();

	const u_char* Data() const	{ return msg_buf; }
	int Len() const			{ return msg_len; }
	bool empty() const		{ return buf_n == 0; }

protected:
	virtual void compute_msg_length() = 0;

	int hdr_len;
	u_char* msg_buf;
	int msg_len;
	int buf_n;	// number of bytes in msg_buf
	int buf_len;	// size off msg_buf
};

#define NCP_TCPIP_HEADER_LENGTH 8

class NCP_FrameBuffer : public FrameBuffer {
public:
	NCP_FrameBuffer() : FrameBuffer(NCP_TCPIP_HEADER_LENGTH) {}

protected:
	void compute_msg_length();
};

class TCP_Contents_NCP : public TCP_Contents {
public:
	TCP_Contents_NCP(TCP_Endpoint* endp, NCP_Session* session);
	~TCP_Contents_NCP();

protected:
	void Deliver(int seq, int len, const u_char* data);
	void Undelivered(int seq, int len);

	NCP_FrameBuffer buffer;
	NCP_Session* session;

	// Re-sync for partial connections (or after a content gap).
	bool resync;
};

class NCP_Conn : public TCP_Connection {
public:
	NCP_Conn(NetSessions* s, HashKey* k, double t, const ConnID* id,
			const struct tcphdr* tp);
	~NCP_Conn();

protected:
	void BuildEndpoints();

	NCP_Session* session;
	TCP_Contents_NCP* o_ncp;
	TCP_Contents_NCP* r_ncp;
};

#endif /* ncp_h */
