%{
// $Id: pac_scan.ll 1436 2005-10-01 00:45:24Z vern $

#include "pac.h"
#include "pac_output.h"
#include "pac_parse.h"
#include "pac_utils.h"

int line_number = 1;
%}

WS	[ \t]+
ID	[A-Za-z_][A-Za-z_0-9]*
D	[0-9]+
HEX	[0-9a-fA-F]+

%option nounput

%%

#.*	/* eat comments */
{WS}	/* eat whitespace */
\n	++line_number;

type	return TOK_TYPE;
record	return TOK_RECORD;
case	return TOK_CASE;
of	return TOK_OF;
default return TOK_DEFAULT;
let	return TOK_LET;
padding	return TOK_PADDING;
to	return TOK_TO;
align	return TOK_ALIGN;
sizeof	return TOK_SIZEOF;
offsetof return TOK_OFFSETOF;
enum	return TOK_ENUM;

&byteorder	return TOK_ATTR_BYTEORDER;
&check		return TOK_ATTR_CHECK;
&endwith	return TOK_ATTR_ENDWITH;
&length		return TOK_ATTR_LENGTH;

"=="	return TOK_EQUAL;
"=>"	return TOK_GE;
"<="	return TOK_LE;
"<<"	return TOK_LSHIFT;
">>"	return TOK_RSHIFT;
"&&"	return TOK_AND;
"||"	return TOK_OR;

"0x"{HEX} 	{
		int n;
		sscanf(yytext + 2, "%x", &n);
		yylval.num = new Number(yytext, n);
		return TOK_NUMBER;
		}

{D} 		{
		int n;
		sscanf(yytext, "%d", &n);
		yylval.num = new Number(yytext, n);
		return TOK_NUMBER;
		}

{ID}		{
		yylval.id = new ID(yytext);
		return TOK_ID;
		}

[\.!%*/+\-&|\^,:;<=>?()\[\]{}~]	return yytext[0];

%%

int yywrap()
	{
	yy_delete_buffer(YY_CURRENT_BUFFER);
	return 1;
	}

void switch_to_file(FILE* fp_input)
	{
	yy_switch_to_buffer(yy_create_buffer(fp_input, YY_BUF_SIZE));
	}
